package com.wechi.adweb.bridge.google.analytics.dto.report.row;

import static com.wechi.adweb.bridge.google.analytics.dto.report.ReportConstant.*;

import com.google.analytics.data.v1beta.Row;
import com.wechi.adweb.bridge.google.analytics.dto.report.GAReportRequestDTO;
import com.wechi.adweb.bridge.util.NumberUtils;

import lombok.Builder;
import lombok.Data;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author wfansh
 */
@Data
@Builder
public class PagePathViewData implements GAReportDataDTO {

    private String pagePath;

    private String engagementRate;

    private int pageViews;

    private BigDecimal avgTimeOnPage;

    public static List<PagePathViewData> fromReportRows(
            List<Row> reportRows, GAReportRequestDTO reportRequest) {
        int indexPagePath = reportRequest.getDimensions().indexOf(DIMENSION_PAGE_PATH);
        int indexEngagementRate = reportRequest.getMetrics().indexOf(METRIC_ENGAGEMENT_RATE);
        int indexPageViews = reportRequest.getMetrics().indexOf(METRIC_SCREEN_PAGE_VIEWS);
        int indexUserEngagementDuration =
                reportRequest.getMetrics().indexOf(METRIC_USER_ENGAGEMENT_DURATION);

        return reportRows.stream()
                .map(
                        row ->
                                PagePathViewData.builder()
                                        .pagePath(row.getDimensionValues(indexPagePath).getValue())
                                        .engagementRate(
                                                NumberUtils.formatPercentage(
                                                        Double.parseDouble(
                                                                row.getMetricValues(
                                                                                indexEngagementRate)
                                                                        .getValue()),
                                                        2))
                                        .pageViews(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexPageViews)
                                                                .getValue()))
                                        .avgTimeOnPage(
                                                NumberUtils.formatDecimal(
                                                        Double.parseDouble(
                                                                row.getMetricValues(
                                                                                indexUserEngagementDuration)
                                                                        .getValue()),
                                                        2))
                                        .build())
                .toList();
    }
}
